@@ -7,6 +7,9 @@ use base 'Mojo::Base';
 
 require Carp;
 
+__PACKAGE__->attr(is_async => 0);
+__PACKAGE__->attr('error');
+
 sub create { Carp::croak('Method "create" not implemented by subbclass') }
 
 sub update { Carp::croak('Method "update" not implemented by subbclass') }
